@@ -56,12 +56,17 @@ extern const isc::log::MessageID DHCP4_DEFERRED_OPTION_MISSING;
 extern const isc::log::MessageID DHCP4_DEFERRED_OPTION_UNPACK_FAIL;
 extern const isc::log::MessageID DHCP4_DEVELOPMENT_VERSION;
 extern const isc::log::MessageID DHCP4_DHCP4O6_BAD_PACKET;
+extern const isc::log::MessageID DHCP4_DHCP4O6_HOOK_SUBNET4_SELECT_DROP;
+extern const isc::log::MessageID DHCP4_DHCP4O6_HOOK_SUBNET4_SELECT_SKIP;
 extern const isc::log::MessageID DHCP4_DHCP4O6_PACKET_RECEIVED;
 extern const isc::log::MessageID DHCP4_DHCP4O6_PACKET_SEND;
 extern const isc::log::MessageID DHCP4_DHCP4O6_PACKET_SEND_FAIL;
 extern const isc::log::MessageID DHCP4_DHCP4O6_RECEIVE_FAIL;
 extern const isc::log::MessageID DHCP4_DHCP4O6_RECEIVING;
 extern const isc::log::MessageID DHCP4_DHCP4O6_RESPONSE_DATA;
+extern const isc::log::MessageID DHCP4_DHCP4O6_SUBNET_DATA;
+extern const isc::log::MessageID DHCP4_DHCP4O6_SUBNET_SELECTED;
+extern const isc::log::MessageID DHCP4_DHCP4O6_SUBNET_SELECTION_FAILED;
 extern const isc::log::MessageID DHCP4_DYNAMIC_RECONFIGURATION;
 extern const isc::log::MessageID DHCP4_DYNAMIC_RECONFIGURATION_FAIL;
 extern const isc::log::MessageID DHCP4_DYNAMIC_RECONFIGURATION_SUCCESS;
@@ -73,6 +78,7 @@ extern const isc::log::MessageID DHCP4_HOOKS_LIBS_RELOAD_FAIL;
 extern const isc::log::MessageID DHCP4_HOOK_BUFFER_RCVD_DROP;
 extern const isc::log::MessageID DHCP4_HOOK_BUFFER_RCVD_SKIP;
 extern const isc::log::MessageID DHCP4_HOOK_BUFFER_SEND_SKIP;
+extern const isc::log::MessageID DHCP4_HOOK_DDNS_UPDATE;
 extern const isc::log::MessageID DHCP4_HOOK_DECLINE_SKIP;
 extern const isc::log::MessageID DHCP4_HOOK_LEASE4_RELEASE_SKIP;
 extern const isc::log::MessageID DHCP4_HOOK_LEASES4_COMMITTED_DROP;
@@ -96,6 +102,8 @@ extern const isc::log::MessageID DHCP4_NO_LEASE_INIT_REBOOT;
 extern const isc::log::MessageID DHCP4_NO_SOCKETS_OPEN;
 extern const isc::log::MessageID DHCP4_OPEN_CONFIG_DB;
 extern const isc::log::MessageID DHCP4_OPEN_SOCKET;
+extern const isc::log::MessageID DHCP4_OPEN_SOCKETS_FAILED;
+extern const isc::log::MessageID DHCP4_OPEN_SOCKETS_NO_RECONNECT_CTL;
 extern const isc::log::MessageID DHCP4_OPEN_SOCKET_FAIL;
 extern const isc::log::MessageID DHCP4_PACKET_DROP_0001;
 extern const isc::log::MessageID DHCP4_PACKET_DROP_0002;
@@ -133,7 +141,9 @@ extern const isc::log::MessageID DHCP4_POST_ALLOCATION_NAME_UPDATE_FAIL;
 extern const isc::log::MessageID DHCP4_QUERY_DATA;
 extern const isc::log::MessageID DHCP4_RECLAIM_EXPIRED_LEASES_FAIL;
 extern const isc::log::MessageID DHCP4_RELEASE;
+extern const isc::log::MessageID DHCP4_RELEASE_DELETED;
 extern const isc::log::MessageID DHCP4_RELEASE_EXCEPTION;
+extern const isc::log::MessageID DHCP4_RELEASE_EXPIRED;
 extern const isc::log::MessageID DHCP4_RELEASE_FAIL;
 extern const isc::log::MessageID DHCP4_RELEASE_FAIL_NO_LEASE;
 extern const isc::log::MessageID DHCP4_RELEASE_FAIL_WRONG_CLIENT;
