@@ -7,6 +7,7 @@
 #include <config.h>
 #include <dhcp/iface_mgr.h>
 #include <dhcp/option_custom.h>
+#include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/cfg_subnets4.h>
 #include <dhcpsrv/dhcpsrv_log.h>
 #include <dhcpsrv/lease_mgr_factory.h>
@@ -224,14 +225,21 @@ CfgSubnets4::initSelector(const Pkt4Ptr& query) {
         OptionCustomPtr rai_custom =
             boost::dynamic_pointer_cast<OptionCustom>(rai);
         if (rai_custom) {
-            OptionPtr link_select =
-                rai_custom->getOption(RAI_OPTION_LINK_SELECTION);
-            if (link_select) {
-                OptionBuffer link_select_buf = link_select->getData();
-                if (link_select_buf.size() == sizeof(uint32_t)) {
-                    selector.option_select_ =
-                        IOAddress::fromBytes(AF_INET, &link_select_buf[0]);
-                    return (selector);
+            // If Relay Agent Information Link Selection is ignored in the
+            // configuration, skip returning the related subnet selector here,
+            // and move on to normal subnet selection.
+            bool ignore_link_sel = CfgMgr::instance().getCurrentCfg()->
+                                   getIgnoreRAILinkSelection();
+            if (!ignore_link_sel) {
+                OptionPtr link_select =
+                    rai_custom->getOption(RAI_OPTION_LINK_SELECTION);
+                if (link_select) {
+                    OptionBuffer link_select_buf = link_select->getData();
+                    if (link_select_buf.size() == sizeof(uint32_t)) {
+                        selector.option_select_ =
+                            IOAddress::fromBytes(AF_INET, &link_select_buf[0]);
+                        return (selector);
+                    }
                 }
             }
         }
@@ -300,7 +308,7 @@ CfgSubnets4::selectSubnet(const SubnetSelector& selector) const {
                              selector.client_classes_));
     } else {
         LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
-                  DHCPSRV_SUBNET4_SELECT_FAILED_NO_RAI_OPTIONS_ADDRESS);
+                  DHCPSRV_SUBNET4_SELECT_NO_RAI_OPTIONS);
     }
 
     // If relayed message has been received, try to match the giaddr with the
@@ -337,11 +345,11 @@ CfgSubnets4::selectSubnet(const SubnetSelector& selector) const {
             }
         }
         LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
-                  DHCPSRV_SUBNET4_SELECT_BY_RELAY_ADDRESS_FAILED)
+                  DHCPSRV_SUBNET4_SELECT_BY_RELAY_ADDRESS_NO_MATCH)
             .arg(selector.giaddr_.toText());
     } else {
         LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
-                  DHCPSRV_SUBNET4_SELECT_FAILED_NO_RELAY_ADDRESS);
+                  DHCPSRV_SUBNET4_SELECT_NO_RELAY_ADDRESS);
     }
 
     // If we got to this point it means that we were not able to match the
@@ -394,7 +402,7 @@ CfgSubnets4::selectSubnet(const SubnetSelector& selector) const {
     // Unable to find a suitable address to use for subnet selection.
     if (address.isV4Zero()) {
         LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
-                  DHCPSRV_SUBNET4_SELECT_FAILED_NO_ADDRESS);
+                  DHCPSRV_SUBNET4_SELECT_NO_USABLE_ADDRESS);
 
         return (Subnet4Ptr());
     }
@@ -441,7 +449,7 @@ CfgSubnets4::selectSubnet(const std::string& iface,
     }
 
     LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
-              DHCPSRV_SUBNET4_SELECT_BY_INTERFACE_FAILED)
+              DHCPSRV_SUBNET4_SELECT_BY_INTERFACE_NO_MATCH)
         .arg(iface);
 
     // Failed to find a subnet.
@@ -480,7 +488,7 @@ CfgSubnets4::selectSubnet(const IOAddress& address,
     }
 
     LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
-              DHCPSRV_SUBNET4_SELECT_BY_ADDRESS_FAILED)
+              DHCPSRV_SUBNET4_SELECT_BY_ADDRESS_NO_MATCH)
         .arg(address.toText());
 
     // Failed to find a subnet.
@@ -527,11 +535,16 @@ CfgSubnets4::updateStatistics() {
                            generateName("subnet", subnet_id, "total-addresses"),
                                         static_cast<int64_t>
                                         (subnet4->getPoolCapacity(Lease::TYPE_V4)));
-        const std::string& name =
+        std::string name =
             StatsMgr::generateName("subnet", subnet_id, "cumulative-assigned-addresses");
         if (!stats_mgr.getObservation(name)) {
             stats_mgr.setValue(name, static_cast<int64_t>(0));
         }
+
+        name = StatsMgr::generateName("subnet", subnet_id, "v4-reservation-conflicts");
+        if (!stats_mgr.getObservation(name)) {
+            stats_mgr.setValue(name, static_cast<int64_t>(0));
+        }
     }
 
     // Only recount the stats if we have subnets.
