@@ -7,8 +7,11 @@
 #include <config.h>
 
 #include <eval/dependency.h>
+#include <eval/evaluate.h>
+#include <eval/eval_log.h>
 #include <dhcpsrv/client_class_def.h>
 #include <dhcpsrv/cfgmgr.h>
+#include <dhcpsrv/dhcpsrv_log.h>
 #include <dhcpsrv/parsers/client_class_def_parser.h>
 #include <boost/foreach.hpp>
 
@@ -137,6 +140,68 @@ ClientClassDef::setCfgOption(const CfgOptionPtr& cfg_option) {
     cfg_option_ = cfg_option;
 }
 
+void
+ClientClassDef::test(PktPtr pkt, const ExpressionPtr& expr_ptr) {
+    // Evaluate the expression which can return false (no match),
+    // true (match) or raise an exception (error)
+    try {
+        bool status = evaluateBool(*expr_ptr, *pkt);
+        if (status) {
+            LOG_INFO(dhcpsrv_logger, EVAL_RESULT)
+                .arg(getName())
+                .arg(status);
+            // Matching: add the class
+            pkt->addClass(getName());
+        } else {
+            LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE_DETAIL, EVAL_RESULT)
+                .arg(getName())
+                .arg(status);
+        }
+    } catch (const Exception& ex) {
+        LOG_ERROR(dhcpsrv_logger, EVAL_RESULT)
+            .arg(getName())
+            .arg(ex.what());
+    } catch (...) {
+        LOG_ERROR(dhcpsrv_logger, EVAL_RESULT)
+            .arg(getName())
+            .arg("get exception?");
+    }
+}
+
+TemplateClientClassDef::TemplateClientClassDef(const std::string& name,
+                                               const ExpressionPtr& match_expr,
+                                               const CfgOptionPtr& options) :
+        ClientClassDef(name, match_expr, options) {
+}
+
+void
+TemplateClientClassDef::test(PktPtr pkt, const ExpressionPtr& expr_ptr) {
+    // Evaluate the expression which can return false (no match),
+    // true (match) or raise an exception (error)
+    try {
+        std::string subclass = evaluateString(*expr_ptr, *pkt);
+        if (!subclass.empty()) {
+            LOG_INFO(dhcpsrv_logger, EVAL_RESULT)
+                .arg(getName())
+                .arg(subclass);
+            // Matching: add the subclass
+            std::string value("SPAWN_");
+            value += getName();
+            value += "_";
+            value += subclass;
+            pkt->addSubClass(getName(), value);
+        }
+    } catch (const Exception& ex) {
+        LOG_ERROR(dhcpsrv_logger, EVAL_RESULT)
+            .arg(getName())
+            .arg(ex.what());
+    } catch (...) {
+        LOG_ERROR(dhcpsrv_logger, EVAL_RESULT)
+            .arg(getName())
+            .arg("get exception?");
+    }
+}
+
 bool
 ClientClassDef::dependOnClass(const std::string& name) const {
     return (isc::dhcp::dependOnClass(match_expr_, name));
@@ -162,7 +227,7 @@ ClientClassDef::equals(const ClientClassDef& other) const {
 }
 
 ElementPtr
-ClientClassDef:: toElement() const {
+ClientClassDef::toElement() const {
     uint16_t family = CfgMgr::instance().getFamily();
     ElementPtr result = Element::createMap();
     // Set user-context
@@ -230,6 +295,19 @@ ClientClassDef:: toElement() const {
     return (result);
 }
 
+ElementPtr
+TemplateClientClassDef::toElement() const {
+    auto const& result = ClientClassDef::toElement();
+    auto const& test = result->get("test");
+    if (test) {
+        result->set("template-test", test);
+        result->remove("test");
+    } else {
+        result->set("template-test", Element::create(""));
+    }
+    return (result);
+}
+
 std::ostream& operator<<(std::ostream& os, const ClientClassDef& x) {
     os << "ClientClassDef:" << x.getName();
     return (os);
@@ -265,8 +343,14 @@ ClientClassDictionary::addClass(const std::string& name,
                                 const std::string& sname,
                                 const std::string& filename,
                                 const util::Triplet<uint32_t>& valid,
-                                const util::Triplet<uint32_t>& preferred) {
-    ClientClassDefPtr cclass(new ClientClassDef(name, match_expr, cfg_option));
+                                const util::Triplet<uint32_t>& preferred,
+                                bool is_template) {
+    ClientClassDefPtr cclass;
+    if (is_template) {
+        cclass.reset(new TemplateClientClassDef(name, match_expr, cfg_option));
+    } else {
+        cclass.reset(new ClientClassDef(name, match_expr, cfg_option));
+    }
     cclass->setTest(test);
     cclass->setRequired(required);
     cclass->setDependOnKnown(depend_on_known);
@@ -394,7 +478,12 @@ ClientClassDictionary::initMatchExpr(uint16_t family) {
         if (!c->getTest().empty()) {
             ExpressionPtr match_expr = boost::make_shared<Expression>();
             ExpressionParser parser;
-            parser.parse(match_expr, Element::create(c->getTest()), family);
+            EvalContext::ParserType parser_type = EvalContext::PARSER_BOOL;
+            if (dynamic_cast<TemplateClientClassDef*>(c.get())) {
+                parser_type = EvalContext::PARSER_STRING;
+            }
+            parser.parse(match_expr, Element::create(c->getTest()), family,
+                         EvalContext::acceptAll, parser_type);
             expressions.push(match_expr);
         }
     }
@@ -472,12 +561,15 @@ ClientClassDictionary::operator=(const ClientClassDictionary& rhs) {
     return (*this);
 }
 
+/// @brief List of classes for which test expressions cannot be defined.
 std::list<std::string>
 builtinNames = {
     // DROP is not in this list because it is special but not built-in.
     // In fact DROP is set from an expression as callouts can drop
     // directly the incoming packet. The expression must not depend on
     // KNOWN/UNKNOWN which are set far after the drop point.
+    // SKIP_DDNS, used by DDNS-tuning is also omitted from this list
+    // so users may assign it a test expression.
     "ALL", "KNOWN", "UNKNOWN", "BOOTP"
 };
 
