@@ -65,6 +65,10 @@ extern const isc::log::MessageID DHCPSRV_HOOK_LEASE6_EXTEND_SKIP;
 extern const isc::log::MessageID DHCPSRV_HOOK_LEASE6_RECOVER_SKIP;
 extern const isc::log::MessageID DHCPSRV_HOOK_LEASE6_SELECT_SKIP;
 extern const isc::log::MessageID DHCPSRV_INVALID_ACCESS;
+extern const isc::log::MessageID DHCPSRV_LEASE4_EXTENDED_INFO_SANITY_FAIL;
+extern const isc::log::MessageID DHCPSRV_LEASE4_EXTENDED_INFO_UPGRADED;
+extern const isc::log::MessageID DHCPSRV_LEASE6_EXTENDED_INFO_SANITY_FAIL;
+extern const isc::log::MessageID DHCPSRV_LEASE6_EXTENDED_INFO_UPGRADED;
 extern const isc::log::MessageID DHCPSRV_LEASE_SANITY_FAIL;
 extern const isc::log::MessageID DHCPSRV_LEASE_SANITY_FAIL_DISCARD;
 extern const isc::log::MessageID DHCPSRV_LEASE_SANITY_FIXED;
@@ -91,8 +95,13 @@ extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_HOSTNAME6;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_HWADDR;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_IAID_DUID;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_IAID_SUBID_DUID;
+extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_LINKADDR6;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_PAGE4;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_PAGE6;
+extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_RELAYID4;
+extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_RELAYID6;
+extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_REMOTEID4;
+extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_REMOTEID6;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_SUBID4;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_SUBID6;
 extern const isc::log::MessageID DHCPSRV_MEMFILE_GET_SUBID_CLIENTID;
@@ -219,15 +228,15 @@ extern const isc::log::MessageID DHCPSRV_QUEUE_NCR;
 extern const isc::log::MessageID DHCPSRV_QUEUE_NCR_FAILED;
 extern const isc::log::MessageID DHCPSRV_QUEUE_NCR_SKIP;
 extern const isc::log::MessageID DHCPSRV_SUBNET4O6_SELECT_FAILED;
-extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_BY_ADDRESS_FAILED;
-extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_BY_INTERFACE_FAILED;
-extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_BY_RELAY_ADDRESS_FAILED;
-extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_FAILED_NO_ADDRESS;
-extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_FAILED_NO_RAI_OPTIONS_ADDRESS;
-extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_FAILED_NO_RELAY_ADDRESS;
-extern const isc::log::MessageID DHCPSRV_SUBNET6_SELECT_BY_ADDRESS_FAILED;
-extern const isc::log::MessageID DHCPSRV_SUBNET6_SELECT_BY_INTERFACE_FAILED;
-extern const isc::log::MessageID DHCPSRV_SUBNET6_SELECT_BY_INTERFACE_ID_FAILED;
+extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_BY_ADDRESS_NO_MATCH;
+extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_BY_INTERFACE_NO_MATCH;
+extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_BY_RELAY_ADDRESS_NO_MATCH;
+extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_NO_RAI_OPTIONS;
+extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_NO_RELAY_ADDRESS;
+extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_NO_USABLE_ADDRESS;
+extern const isc::log::MessageID DHCPSRV_SUBNET6_SELECT_BY_ADDRESS_NO_MATCH;
+extern const isc::log::MessageID DHCPSRV_SUBNET6_SELECT_BY_INTERFACE_ID_NO_MATCH;
+extern const isc::log::MessageID DHCPSRV_SUBNET6_SELECT_BY_INTERFACE_NO_MATCH;
 extern const isc::log::MessageID DHCPSRV_TIMERMGR_CALLBACK_FAILED;
 extern const isc::log::MessageID DHCPSRV_TIMERMGR_REGISTER_TIMER;
 extern const isc::log::MessageID DHCPSRV_TIMERMGR_RUN_TIMER_OPERATION;
