@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -41,6 +41,7 @@ Lease::Lease(const isc::asiolink::IOAddress& addr,
              const bool fqdn_fwd, const bool fqdn_rev,
              const std::string& hostname, const HWAddrPtr& hwaddr)
     : addr_(addr), valid_lft_(valid_lft), current_valid_lft_(valid_lft),
+      reuseable_valid_lft_(0),
       cltt_(cltt), current_cltt_(cltt), subnet_id_(subnet_id),
       hostname_(boost::algorithm::to_lower_copy(hostname)), fqdn_fwd_(fqdn_fwd),
       fqdn_rev_(fqdn_rev), hwaddr_(hwaddr), state_(STATE_DEFAULT) {
@@ -172,7 +173,10 @@ Lease::fromElementCommon(const LeasePtr& lease, const data::ConstElementPtr& ele
 
     if (subnet_id->intValue() <= 0) {
         isc_throw(BadValue, "subnet-id " << subnet_id->intValue() << " is not"
-                            << " a positive integer");
+                  << " a positive integer");
+    } else if (subnet_id->intValue() > numeric_limits<uint32_t>::max()) {
+        isc_throw(BadValue, "subnet-id " << subnet_id->intValue() << " is not"
+                  << " a 32 bit unsigned integer");
     }
 
     lease->subnet_id_ = SubnetID(subnet_id->intValue());
@@ -339,7 +343,7 @@ Lease4::statesToText(const uint32_t state) {
 
 const std::vector<uint8_t>&
 Lease4::getClientIdVector() const {
-    if(!client_id_) {
+    if (!client_id_) {
         static std::vector<uint8_t> empty_vec;
         return (empty_vec);
     }
@@ -391,6 +395,7 @@ Lease4::operator=(const Lease4& other) {
         addr_ = other.addr_;
         valid_lft_ = other.valid_lft_;
         current_valid_lft_ = other.current_valid_lft_;
+        reuseable_valid_lft_ = other.reuseable_valid_lft_;
         cltt_ = other.cltt_;
         current_cltt_ = other.current_cltt_;
         subnet_id_ = other.subnet_id_;
@@ -487,7 +492,7 @@ Lease6::Lease6(Lease::Type type, const isc::asiolink::IOAddress& addr,
                SubnetID subnet_id, const HWAddrPtr& hwaddr, uint8_t prefixlen)
     : Lease(addr, valid, subnet_id, 0/*cltt*/, false, false, "", hwaddr),
       type_(type), prefixlen_(prefixlen), iaid_(iaid), duid_(duid),
-      preferred_lft_(preferred) {
+      preferred_lft_(preferred), reuseable_preferred_lft_(0) {
     if (!duid) {
         isc_throw(InvalidOperation, "DUID is mandatory for an IPv6 lease");
     }
@@ -504,7 +509,7 @@ Lease6::Lease6(Lease::Type type, const isc::asiolink::IOAddress& addr,
     : Lease(addr, valid, subnet_id, 0/*cltt*/,
             fqdn_fwd, fqdn_rev, hostname, hwaddr),
       type_(type), prefixlen_(prefixlen), iaid_(iaid), duid_(duid),
-      preferred_lft_(preferred) {
+      preferred_lft_(preferred), reuseable_preferred_lft_(0) {
     if (!duid) {
         isc_throw(InvalidOperation, "DUID is mandatory for an IPv6 lease");
     }
@@ -516,7 +521,7 @@ Lease6::Lease6(Lease::Type type, const isc::asiolink::IOAddress& addr,
 Lease6::Lease6()
     : Lease(isc::asiolink::IOAddress("::"), 0, 0, 0, false, false, "",
             HWAddrPtr()), type_(TYPE_NA), prefixlen_(0), iaid_(0),
-            duid_(DuidPtr()), preferred_lft_(0) {
+            duid_(DuidPtr()), preferred_lft_(0), reuseable_preferred_lft_(0) {
 }
 
 std::string
@@ -600,6 +605,7 @@ Lease4::operator==(const Lease4& other) const {
             subnet_id_ == other.subnet_id_ &&
             valid_lft_ == other.valid_lft_ &&
             current_valid_lft_ == other.current_valid_lft_ &&
+            reuseable_valid_lft_ == other.reuseable_valid_lft_ &&
             cltt_ == other.cltt_ &&
             current_cltt_ == other.current_cltt_ &&
             hostname_ == other.hostname_ &&
@@ -618,8 +624,10 @@ Lease6::operator==(const Lease6& other) const {
             prefixlen_ == other.prefixlen_ &&
             iaid_ == other.iaid_ &&
             preferred_lft_ == other.preferred_lft_ &&
+            reuseable_preferred_lft_ == other.reuseable_preferred_lft_ &&
             valid_lft_ == other.valid_lft_ &&
             current_valid_lft_ == other.current_valid_lft_ &&
+            reuseable_valid_lft_ == other.reuseable_valid_lft_ &&
             cltt_ == other.cltt_ &&
             current_cltt_ == other.current_cltt_ &&
             subnet_id_ == other.subnet_id_ &&
