@@ -10,10 +10,12 @@
 #include <asiolink/io_address.h>
 #include <asiolink/io_service.h>
 #include <cc/data.h>
+#include <database/database_connection.h>
 #include <database/db_exceptions.h>
 #include <dhcp/duid.h>
 #include <dhcp/option.h>
 #include <dhcp/hwaddr.h>
+#include <dhcpsrv/cfg_consistency.h>
 #include <dhcpsrv/lease.h>
 #include <dhcpsrv/subnet.h>
 
@@ -224,7 +226,7 @@ class LeaseMgr {
 public:
     /// @brief Constructor
     ///
-    LeaseMgr()
+    LeaseMgr() : extended_info_tables_enabled_(false)
     {}
 
     /// @brief Destructor
@@ -813,9 +815,45 @@ public:
     /// @brief Clears the class-lease count map.
     virtual void clearClassLeaseCounts() = 0;
 
-    /// The following queries are used to fulfill Bulk LeaseQuery queries. They rely
-    /// on relay data contained in lease's user-context when the extended-store-info flag
-    /// is enabled.
+    /// The following queries are used to fulfill Bulk Lease Query
+    /// queries. They rely on relay data contained in lease's
+    /// user-context when the extended-store-info flag is enabled.
+
+    /// @brief Upgrade a V4 lease user context to the new extended info entry.
+    ///
+    /// In details:
+    ///  - perform sanity checks according to check level.
+    ///  - change the "ISC" / "relay-agent-info" to a map.
+    ///  - move the "relay-agent-info" string to the "sub-options" entry of
+    ///    the map.
+    ///  - decode remote-id and relay-id from the RAI option content and
+    ///    add the raw value in hexadecimal in "remote-id" and/or "relay-id"
+    ///    entries of the map.
+    ///
+    /// @param lease Pointer to the lease to be updated.
+    /// @param check Sanity/consistency check level.
+    /// @return True if the lease user context was updated, false otherwise.
+    static bool
+    upgradeLease4ExtendedInfo(const Lease4Ptr& lease,
+                              CfgConsistency::ExtendedInfoSanity check =
+                              CfgConsistency::EXTENDED_INFO_CHECK_FIX);
+
+    /// @brief Upgrade a V6 lease user context to the new extended info entry.
+    ///
+    /// In details:
+    ///  - perform sanity checks according to check level.
+    ///  - change the "ISC" / "relays" list entry to "relay-info".
+    ///  - decode remote-id and relay-id from each relay options and
+    ///    add the raw value in hexadecimal in "remote-id" and/or "relay-id"
+    ///    in the relay item of the list.
+    ///
+    /// @param lease Pointer to the lease to be updated.
+    /// @param check Sanity/consistency check level.
+    /// @return True if the lease user context was updated, false otherwise.
+    static bool
+    upgradeLease6ExtendedInfo(const Lease6Ptr& lease,
+                              CfgConsistency::ExtendedInfoSanity check =
+                              CfgConsistency::EXTENDED_INFO_CHECK_FIX);
 
     /// @brief Returns existing IPv4 leases with a given relay-id.
     ///
@@ -887,7 +925,7 @@ public:
 
     /// @brief Returns existing IPv6 leases with on a given link.
     ///
-    /// @param link_addr limit results to leases on this link when not ::
+    /// @param link_addr limit results to leases on this link.
     /// @param lower_bound_address IPv4 address used as lower bound for the
     /// returned range.
     /// @param page_size maximum size of the page returned.
@@ -908,9 +946,75 @@ public:
     /// @param filename File name to write leases.
     virtual void writeLeases6(const std::string& filename) = 0;
 
+    /// @brief Returns the setting indicating if lease extended info tables
+    /// are enabled.
+    ///
+    /// @return true if lease extended info tables are enabled or false
+    /// if they are disabled.
+    bool getExtendedInfoTablesEnabled() const {
+        return (extended_info_tables_enabled_);
+    }
+
+protected:
+
+    /// Extended information / Bulk Lease Query shared interface.
+
+    /// @brief Modifies the setting whether the lease extended info tables
+    /// are enabled.
+    ///
+    /// @note This method is virtual so backend doing specific action
+    /// on value changes can intercept it by redefining it.
+    ///
+    /// @param enabled new setting.
+    virtual void setExtendedInfoTablesEnabled(const bool enabled) {
+        extended_info_tables_enabled_ = enabled;
+    }
+
+    /// @brief Decode parameters to set whether the lease extended info tables
+    /// are enabled.
+    ///
+    /// @note: common code in constructors.
+    ///
+    /// @param parameters The parameter map.
+    virtual void setExtendedInfoTablesEnabled(const db::DatabaseConnection::ParameterMap& parameters);
+
+    /// @brief Delete lease6 extended info from tables.
+    ///
+    /// @param addr The address of the lease.
+    virtual void deleteExtendedInfo6(const isc::asiolink::IOAddress& addr) = 0;
+
+    /// @brief Add lease6 extended info into by-relay-id table.
+    ///
+    /// @param lease_addr The address of the lease.
+    /// @param link_addr The link address from the relay header.
+    /// @param relay_id The relay id from the relay header options.
+    virtual void addRelayId6(const isc::asiolink::IOAddress& lease_addr,
+                             const isc::asiolink::IOAddress& link_addr,
+                             const std::vector<uint8_t>& relay_id) = 0;
+
+    /// @brief Add lease6 extended info into by-remote-id table.
+    ///
+    /// @param lease_addr The address of the lease.
+    /// @param link_addr The link address from the remote header.
+    /// @param remote_id The remote id from the relay header options.
+    virtual void addRemoteId6(const isc::asiolink::IOAddress& lease_addr,
+                              const isc::asiolink::IOAddress& link_addr,
+                              const std::vector<uint8_t>& remote_id) = 0;
+
+    /// @brief Add lease6 extended info into by-link-addr table.
+    ///
+    /// @param lease_addr The address of the lease.
+    /// @param link_addr The link address from the remote header.
+    virtual void addLinkAddr6(const isc::asiolink::IOAddress& lease_addr,
+                              const isc::asiolink::IOAddress& link_addr) = 0;
+
 private:
     /// The IOService object, used for all ASIO operations.
     static isc::asiolink::IOServicePtr io_service_;
+
+    /// @brief Holds the setting whether the lease extended info tables
+    /// are enabled or disabled. The default is disabled.
+    bool extended_info_tables_enabled_;
 };
 
 }  // namespace dhcp
