@@ -1018,9 +1018,9 @@ private:
                     int status, StatementIndex index,
                     const char* what) const;
 
-    /// The following queries are used to fulfill Bulk LeaseQuery queries. They rely
-    /// on relay data contained in lease's user-context when the extended-store-info flag
-    /// is enabled.
+    /// The following queries are used to fulfill Bulk Lease Query
+    /// queries. They rely on relay data contained in lease's
+    /// user-context when the extended-store-info flag is enabled.
 
     /// @brief Returns existing IPv4 leases with a given relay-id.
     ///
@@ -1092,7 +1092,7 @@ private:
 
     /// @brief Returns existing IPv6 leases with on a given link.
     ///
-    /// @param link_addr limit results to leases on this link when not ::
+    /// @param link_addr limit results to leases on this link.
     /// @param lower_bound_address IPv4 address used as lower bound for the
     /// returned range.
     /// @param page_size maximum size of the page returned.
@@ -1129,6 +1129,60 @@ private:
         const MySqlLeaseMgr& mgr_;
     };
 
+protected:
+
+    /// Extended information / Bulk Lease Query shared interface.
+
+    /// @brief Modifies the setting whether the lease extended info tables
+    /// are enabled.
+    ///
+    /// Transient redefine to refuse the enable setting.
+    /// @param enabled new setting.
+    virtual void setExtendedInfoTablesEnabled(const bool enabled) override {
+        if (enabled) {
+            isc_throw(isc::NotImplemented,
+                      "extended info tables are not yet supported by mysql");
+        }
+    }
+
+    /// @brief Decode parameters to set whether the lease extended info tables
+    /// are enabled.
+    ///
+    /// @note: common code in constructors.
+    ///
+    /// @param parameters The parameter map.
+    virtual void setExtendedInfoTablesEnabled(const db::DatabaseConnection::ParameterMap& parameters) override;
+
+    /// @brief Delete lease6 extended info from tables.
+    ///
+    /// @param addr The address of the lease.
+    virtual void deleteExtendedInfo6(const isc::asiolink::IOAddress& addr) override;
+
+    /// @brief Add lease6 extended info into by-relay-id table.
+    ///
+    /// @param lease_addr The address of the lease.
+    /// @param link_addr The link address from the relay header.
+    /// @param relay_id The relay id from the relay header options.
+    virtual void addRelayId6(const isc::asiolink::IOAddress& lease_addr,
+                             const isc::asiolink::IOAddress& link_addr,
+                             const std::vector<uint8_t>& relay_id) override;
+
+    /// @brief Add lease6 extended info into by-remote-id table.
+    ///
+    /// @param lease_addr The address of the lease.
+    /// @param link_addr The link address from the remote header.
+    /// @param remote_id The remote id from the relay header options.
+    virtual void addRemoteId6(const isc::asiolink::IOAddress& lease_addr,
+                              const isc::asiolink::IOAddress& link_addr,
+                              const std::vector<uint8_t>& remote_id) override;
+
+    /// @brief Add lease6 extended info into by-link-addr table.
+    ///
+    /// @param lease_addr The address of the lease.
+    /// @param link_addr The link address from the remote header.
+    virtual void addLinkAddr6(const isc::asiolink::IOAddress& lease_addr,
+                              const isc::asiolink::IOAddress& link_addr) override;
+
 private:
 
     // Members
