@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -14,13 +14,21 @@
 #include <cc/user_context.h>
 #include <dhcp/classify.h>
 #include <dhcp/option.h>
+#include <dhcpsrv/cfg_globals.h>
 #include <dhcpsrv/cfg_option.h>
 #include <dhcpsrv/cfg_4o6.h>
 #include <dhcpsrv/d2_client_cfg.h>
 #include <util/triplet.h>
 #include <util/optional.h>
+
+#include <boost/multi_index/hashed_index.hpp>
+#include <boost/multi_index/identity.hpp>
+#include <boost/multi_index/indexed_by.hpp>
+#include <boost/multi_index/sequenced_index.hpp>
+#include <boost/multi_index_container.hpp>
 #include <boost/shared_ptr.hpp>
 #include <boost/weak_ptr.hpp>
+
 #include <cstdint>
 #include <functional>
 #include <string>
@@ -34,9 +42,31 @@ inline void unused(Args const& ...) {}
 namespace isc {
 namespace dhcp {
 
-/// List of IOAddresses
-typedef std::vector<isc::asiolink::IOAddress> IOAddressList;
+/// @brief Tag for the list of IO addresses as a list.
+struct IOAddressListListTag { };
 
+/// @brief Tag for the list of IO addresses as a set.
+struct IOAddressListSetTag { };
+
+/// @brief List of IO addresses
+typedef boost::multi_index_container<
+    // Multi index container holds IO addresses.
+    asiolink::IOAddress,
+    // Indexes.
+    boost::multi_index::indexed_by<
+        // First and default index allows for in order iteration.
+        boost::multi_index::sequenced<
+            boost::multi_index::tag<IOAddressListListTag>
+        >,
+        // Second index allows for checking existence.
+        boost::multi_index::hashed_unique<
+            boost::multi_index::tag<IOAddressListSetTag>,
+            boost::multi_index::identity<asiolink::IOAddress>
+        >
+    >
+> IOAddressList;
+
+// @brief Forward declaration of the Network class.
 class Network;
 
 /// @brief Pointer to the @ref Network object.
@@ -47,7 +77,7 @@ typedef boost::weak_ptr<Network> WeakNetworkPtr;
 
 /// @brief Callback function for @c Network that retrieves globally
 /// configured parameters.
-typedef std::function<data::ConstElementPtr()> FetchNetworkGlobalsFn;
+typedef std::function<ConstCfgGlobalsPtr()> FetchNetworkGlobalsFn;
 
 /// @brief Common interface representing a network to which the DHCP clients
 /// are connected.
@@ -219,17 +249,22 @@ public:
     /// If the interface is specified, the server will use the network
     /// associated with this local interface to allocate IP addresses and
     /// other resources to a client.
+    /// Empty values are translated into unspecified.
     ///
     /// @param iface_name Interface name.
     void setIface(const util::Optional<std::string>& iface_name) {
-        iface_name_ = iface_name;
+        if (iface_name.empty()) {
+            iface_name_ = util::Optional<std::string>("", true);
+        } else {
+            iface_name_ = iface_name;
+        }
     }
 
     /// @brief Returns name of the local interface for which this network is
     /// selected.
     ///
     /// @param inheritance inheritance mode to be used.
-    /// @return Interface name as text.
+    /// @return Interface name as optional text.
     util::Optional<std::string>
     getIface(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getIface, iface_name_,
@@ -339,8 +374,9 @@ public:
     /// @param inheritance inheritance mode to be used.
     isc::util::Triplet<uint32_t> getValid(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getValid, valid_, inheritance,
-                                     "valid-lifetime", "min-valid-lifetime",
-                                     "max-valid-lifetime"));
+                                     CfgGlobals::VALID_LIFETIME,
+                                     CfgGlobals::MIN_VALID_LIFETIME,
+                                     CfgGlobals::MAX_VALID_LIFETIME));
     }
 
     /// @brief Sets new valid lifetime for a network.
@@ -354,7 +390,8 @@ public:
     ///
     /// @param inheritance inheritance mode to be used.
     isc::util::Triplet<uint32_t> getT1(const Inheritance& inheritance = Inheritance::ALL) const {
-        return (getProperty<Network>(&Network::getT1, t1_, inheritance, "renew-timer"));
+        return (getProperty<Network>(&Network::getT1, t1_, inheritance,
+                                     CfgGlobals::RENEW_TIMER));
     }
 
     /// @brief Sets new renew timer for a network.
@@ -368,7 +405,8 @@ public:
     ///
     /// @param inheritance inheritance mode to be used.
     isc::util::Triplet<uint32_t> getT2(const Inheritance& inheritance = Inheritance::ALL) const {
-        return (getProperty<Network>(&Network::getT2, t2_, inheritance, "rebind-timer"));
+        return (getProperty<Network>(&Network::getT2, t2_, inheritance,
+                                     CfgGlobals::REBIND_TIMER));
     }
 
     /// @brief Sets new rebind timer for a network.
@@ -386,7 +424,7 @@ public:
         return (getProperty<Network>(&Network::getReservationsGlobal,
                                      reservations_global_,
                                      inheritance,
-                                     "reservations-global"));
+                                     CfgGlobals::RESERVATIONS_GLOBAL));
     }
 
     /// @brief Sets whether global reservations should be fetched.
@@ -404,7 +442,7 @@ public:
         return (getProperty<Network>(&Network::getReservationsInSubnet,
                                      reservations_in_subnet_,
                                      inheritance,
-                                     "reservations-in-subnet"));
+                                     CfgGlobals::RESERVATIONS_IN_SUBNET));
     }
 
     /// @brief Sets whether subnet reservations should be fetched.
@@ -422,7 +460,7 @@ public:
         return (getProperty<Network>(&Network::getReservationsOutOfPool,
                                      reservations_out_of_pool_,
                                      inheritance,
-                                     "reservations-out-of-pool"));
+                                     CfgGlobals::RESERVATIONS_OUT_OF_POOL));
     }
 
     /// @brief Sets whether only out-of-pool reservations are allowed.
@@ -451,7 +489,7 @@ public:
         return (getProperty<Network>(&Network::getCalculateTeeTimes,
                                      calculate_tee_times_,
                                      inheritance,
-                                     "calculate-tee-times"));
+                                     CfgGlobals::CALCULATE_TEE_TIMES));
     }
 
     /// @brief Sets whether or not T1/T2 calculation is enabled.
@@ -467,7 +505,7 @@ public:
     util::Optional<double>
     getT1Percent(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getT1Percent, t1_percent_,
-                                     inheritance, "t1-percent"));
+                                     inheritance, CfgGlobals::T1_PERCENT));
     }
 
     /// @brief Sets new percentage for calculating T1 (renew timer).
@@ -483,7 +521,7 @@ public:
     util::Optional<double>
     getT2Percent(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getT2Percent, t2_percent_,
-                                     inheritance, "t2-percent"));
+                                     inheritance, CfgGlobals::T2_PERCENT));
     }
 
     /// @brief Sets new percentage for calculating T2 (rebind timer).
@@ -498,8 +536,9 @@ public:
     /// @param inheritance inheritance mode to be used.
     util::Optional<bool>
     getDdnsSendUpdates(const Inheritance& inheritance = Inheritance::ALL) const {
-        return (getProperty<Network>(&Network::getDdnsSendUpdates, ddns_send_updates_,
-                                     inheritance, "ddns-send-updates"));
+        return (getProperty<Network>(&Network::getDdnsSendUpdates,
+                                     ddns_send_updates_, inheritance,
+                                     CfgGlobals::DDNS_SEND_UPDATES));
     }
 
     /// @brief Sets new ddns-send-updates
@@ -515,8 +554,8 @@ public:
     util::Optional<bool>
     getDdnsOverrideNoUpdate(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getDdnsOverrideNoUpdate,
-                                     ddns_override_no_update_,
-                                     inheritance, "ddns-override-no-update"));
+                                     ddns_override_no_update_, inheritance,
+                                     CfgGlobals::DDNS_OVERRIDE_NO_UPDATE));
     }
 
     /// @brief Sets new ddns-override-no-update
@@ -532,8 +571,8 @@ public:
     util::Optional<bool>
     getDdnsOverrideClientUpdate(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getDdnsOverrideClientUpdate,
-                                     ddns_override_client_update_,
-                                     inheritance, "ddns-override-client-update"));
+                                     ddns_override_client_update_, inheritance,
+                                     CfgGlobals::DDNS_OVERRIDE_CLIENT_UPDATE));
     }
 
     /// @brief Sets new ddns-override-client-update
@@ -563,7 +602,8 @@ public:
             (inheritance != Inheritance::PARENT_NETWORK)) {
             // Get global mode.
             util::Optional<std::string> mode_label;
-            mode_label = getGlobalProperty(mode_label, "ddns-replace-client-name");
+            mode_label = getGlobalProperty(mode_label,
+                                           CfgGlobals::DDNS_REPLACE_CLIENT_NAME);
             if (!mode_label.unspecified()) {
                 try {
                     // If the mode is globally configured, convert it to an enum.
@@ -595,8 +635,8 @@ public:
     util::Optional<std::string>
     getDdnsGeneratedPrefix(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getDdnsGeneratedPrefix,
-                                     ddns_generated_prefix_,
-                                     inheritance, "ddns-generated-prefix"));
+                                     ddns_generated_prefix_, inheritance,
+                                     CfgGlobals::DDNS_GENERATED_PREFIX));
     }
 
     /// @brief Sets new ddns-generated-prefix
@@ -612,8 +652,8 @@ public:
     util::Optional<std::string>
     getDdnsQualifyingSuffix(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getDdnsQualifyingSuffix,
-                                     ddns_qualifying_suffix_,
-                                     inheritance, "ddns-qualifying-suffix"));
+                                     ddns_qualifying_suffix_, inheritance,
+                                     CfgGlobals::DDNS_QUALIFYING_SUFFIX));
     }
 
     /// @brief Sets new ddns-qualifying-suffix
@@ -626,8 +666,9 @@ public:
     /// @brief Return the char set regexp used to sanitize client hostnames.
     util::Optional<std::string>
     getHostnameCharSet(const Inheritance& inheritance = Inheritance::ALL) const {
-        return (getProperty<Network>(&Network::getHostnameCharSet, hostname_char_set_,
-                                     inheritance, "hostname-char-set"));
+        return (getProperty<Network>(&Network::getHostnameCharSet,
+                                     hostname_char_set_, inheritance,
+                                     CfgGlobals::HOSTNAME_CHAR_SET));
     }
 
     /// @brief Sets new hostname-char-set
@@ -641,8 +682,8 @@ public:
     util::Optional<std::string>
     getHostnameCharReplacement(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getHostnameCharReplacement,
-                                     hostname_char_replacement_,
-                                     inheritance, "hostname-char-replacement"));
+                                     hostname_char_replacement_, inheritance,
+                                     CfgGlobals::HOSTNAME_CHAR_REPLACEMENT));
     }
 
     /// @brief Sets new hostname-char-replacement
@@ -659,8 +700,8 @@ public:
     util::Optional<bool>
     getStoreExtendedInfo(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getStoreExtendedInfo,
-                                     store_extended_info_,
-                                     inheritance, "store-extended-info"));
+                                     store_extended_info_, inheritance,
+                                     CfgGlobals::STORE_EXTENDED_INFO));
     }
 
     /// @brief Sets new store-extended-info
@@ -676,8 +717,8 @@ public:
     util::Optional<double>
     getCacheThreshold(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getCacheThreshold,
-                                     cache_threshold_,
-                                     inheritance, "cache-threshold"));
+                                     cache_threshold_, inheritance,
+                                     CfgGlobals::CACHE_THRESHOLD));
     }
 
     /// @brief Sets cache threshold for a network.
@@ -693,7 +734,7 @@ public:
     util::Optional<uint32_t>
     getCacheMaxAge(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getCacheMaxAge, cache_max_age_,
-                                     inheritance, "cache-max-age"));
+                                     inheritance, CfgGlobals::CACHE_MAX_AGE));
     }
 
     /// @brief Sets cache max for a network.
@@ -709,8 +750,8 @@ public:
     util::Optional<bool>
     getDdnsUpdateOnRenew(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getDdnsUpdateOnRenew,
-                                     ddns_update_on_renew_,
-                                     inheritance, "ddns-update-on-renew"));
+                                     ddns_update_on_renew_, inheritance,
+                                     CfgGlobals::DDNS_UPDATE_ON_RENEW));
     }
 
     /// @brief Sets new ddns-update-on-renew
@@ -727,7 +768,8 @@ public:
     getDdnsUseConflictResolution(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network>(&Network::getDdnsUseConflictResolution,
                                      ddns_use_conflict_resolution_,
-                                     inheritance, "ddns-use-conflict-resolution"));
+                                     inheritance,
+                                     CfgGlobals::DDNS_USE_CONFLICT_RESOLUTION));
     }
 
     /// @brief Sets new ddns-use-conflict-resolution
@@ -753,11 +795,11 @@ protected:
     }
 
     /// @brief Returns a value of global configuration parameter with
-    /// a given name.
+    /// a given index.
     ///
     /// If the @c ferch_globals_fn_ function is non-null, this method will
     /// invoke this function to retrieve a global value having the given
-    /// name. Typically, this method is invoked by @c getProperty when
+    /// index. Typically, this method is invoked by @c getProperty when
     /// network specific value of the parameter is not found. In some cases
     /// it may be called by other methods. One such example is the
     /// @c getDdnsReplaceClientNameMode which needs to call @c getGlobalProperty
@@ -769,25 +811,25 @@ protected:
     ///
     /// @param property Value to be returned when it is specified or when
     /// no global value is found.
-    /// @param global_name Name of the global parameter which value should
+    /// @param global_index Index of the global parameter which value should
     /// be returned
-    /// @param min_name Name of the min global parameter which value should
+    /// @param min_index Index of the min global parameter which value should
     /// be returned for triplets
-    /// @param max_name Name of the max global parameter which value should
+    /// @param max_index Index of the max global parameter which value should
     /// be returned for triplets
     ///
     /// @return Optional value fetched from the global level or the value
     /// of @c property.
     template<typename ReturnType>
     ReturnType getGlobalProperty(ReturnType property,
-                                 const std::string& global_name,
-                                 const std::string& min_name = "",
-                                 const std::string& max_name = "") const {
-        unused(min_name, max_name);
-        if (!global_name.empty() && fetch_globals_fn_) {
-            data::ConstElementPtr globals = fetch_globals_fn_();
-            if (globals && (globals->getType() == data::Element::map)) {
-                data::ConstElementPtr global_param = globals->get(global_name);
+                                 const int global_index,
+                                 const int min_index = -1,
+                                 const int max_index = -1) const {
+        unused(min_index, max_index);
+        if ((global_index >= 0) && fetch_globals_fn_) {
+            ConstCfgGlobalsPtr globals = fetch_globals_fn_();
+            if (globals) {
+                data::ConstElementPtr global_param = globals->get(global_index);
                 if (global_param) {
                     // If there is a global parameter, convert it to the
                     // optional value of the given type and return.
@@ -807,37 +849,37 @@ protected:
     ///
     /// @param property Value to be returned when it is specified or when
     /// no global value is found.
-    /// @param global_name Name of the global parameter which value should
+    /// @param global_index Index of the global parameter which value should
     /// be returned
-    /// @param min_name Name of the min global parameter which value should
+    /// @param min_index Index of the min global parameter which value should
     /// be returned for triplets
-    /// @param max_name Name of the max global parameter which value should
+    /// @param max_index Index of the max global parameter which value should
     /// be returned for triplets
     ///
     /// @return Optional value fetched from the global level or the value
     /// of @c property.
     template<typename NumType>
     isc::util::Triplet<NumType> getGlobalProperty(isc::util::Triplet<NumType> property,
-                                       const std::string& global_name,
-                                       const std::string& min_name = "",
-                                       const std::string& max_name = "") const {
+                                                  const int global_index,
+                                                  const int min_index = -1,
+                                                  const int max_index = -1) const {
 
-        if (!global_name.empty() && fetch_globals_fn_) {
-            data::ConstElementPtr globals = fetch_globals_fn_();
-            if (globals && (globals->getType() == data::Element::map)) {
-                data::ConstElementPtr param = globals->get(global_name);
+        if ((global_index >= 0) && fetch_globals_fn_) {
+            ConstCfgGlobalsPtr globals = fetch_globals_fn_();
+            if (globals) {
+                data::ConstElementPtr param = globals->get(global_index);
                 if (param) {
                     NumType def_value = static_cast<NumType>(param->intValue());
-                    if (min_name.empty() || max_name.empty()) {
+                    if ((min_index < 0) || (max_index < 0)) {
                         return (def_value);
                     } else {
                         NumType min_value = def_value;
                         NumType max_value = def_value;
-                        data::ConstElementPtr min_param = globals->get(min_name);
+                        data::ConstElementPtr min_param = globals->get(min_index);
                         if (min_param) {
                             min_value = static_cast<NumType>(min_param->intValue());
                         }
-                        data::ConstElementPtr max_param = globals->get(max_name);
+                        data::ConstElementPtr max_param = globals->get(max_index);
                         if (max_param) {
                             max_value = static_cast<NumType>(max_param->intValue());
                         }
@@ -860,20 +902,20 @@ protected:
     ///
     /// @param property Value to be returned when it is specified or when
     /// no global value is found.
-    /// @param global_name Name of the global parameter which value should
+    /// @param global_index Index of the global parameter which value should
     /// be returned
-    /// @param min_name Name of the min global parameter which value should
+    /// @param min_index Index of the min global parameter which value should
     /// be returned for triplets
-    /// @param max_name Name of the max global parameter which value should
+    /// @param max_index Index of the max global parameter which value should
     /// be returned for triplets
     ///
     /// @return Optional value fetched from the global level or the value
     /// of @c property.
     util::Optional<asiolink::IOAddress>
     getGlobalProperty(util::Optional<asiolink::IOAddress> property,
-                      const std::string& global_name,
-                      const std::string& min_name = "",
-                      const std::string& max_name = "") const;
+                      const int global_index,
+                      const int min_index = -1,
+                      const int max_index = -1) const;
 
     /// @brief Returns a value associated with a network using inheritance.
     ///
@@ -893,14 +935,14 @@ protected:
     /// @param property Value to be returned when it is specified or when
     /// no explicit value is specified on upper inheritance levels.
     /// @param inheritance inheritance mode to be used.
-    /// @param global_name Optional name of the global parameter which value
+    /// @param global_index Optional index of the global parameter which value
     /// should be returned if the given parameter is not specified on network
     /// level. This value is empty by default, which indicates that the
     /// global value for the given parameter is not supported and shouldn't
     /// be fetched.
-    /// @param min_name Name of the min global parameter which value should
+    /// @param min_index Index of the min global parameter which value should
     /// be returned for triplets
-    /// @param max_name Name of the max global parameter which value should
+    /// @param max_index Index of the max global parameter which value should
     /// be returned for triplets
     ///
     /// @return Optional value fetched from this instance level, parent
@@ -909,9 +951,9 @@ protected:
     ReturnType getProperty(ReturnType(BaseType::*MethodPointer)(const Inheritance&) const,
                            ReturnType property,
                            const Inheritance& inheritance,
-                           const std::string& global_name = "",
-                           const std::string& min_name = "",
-                           const std::string& max_name = "") const {
+                           const int global_index = -1,
+                           const int min_index = -1,
+                           const int max_index = -1) const {
 
         // If no inheritance is to be used, return the value for this
         // network regardless if it is specified or not.
@@ -930,7 +972,7 @@ protected:
 
         // If global value requested, return it.
         } else if (inheritance == Inheritance::GLOBAL) {
-            return (getGlobalProperty(ReturnType(), global_name, min_name, max_name));
+            return (getGlobalProperty(ReturnType(), global_index, min_index, max_index));
         }
 
         // We use inheritance and the value is not specified on the network level.
@@ -953,7 +995,7 @@ protected:
             // can be specified on global level and there is a callback
             // that returns the global values, try to find this parameter
             // at the global scope.
-            return (getGlobalProperty(property, global_name, min_name, max_name));
+            return (getGlobalProperty(property, global_index, min_index, max_index));
         }
 
         // We haven't found the value at any level, so return the unspecified.
@@ -1154,7 +1196,7 @@ public:
         return (getProperty<Network4>(&Network4::getMatchClientId,
                                       match_client_id_,
                                       inheritance,
-                                      "match-client-id"));
+                                      CfgGlobals::MATCH_CLIENT_ID));
     }
 
     /// @brief Sets the flag indicating if the client identifier should be
@@ -1174,8 +1216,9 @@ public:
     /// false otherwise.
     util::Optional<bool>
     getAuthoritative(const Inheritance& inheritance = Inheritance::ALL) const {
-        return (getProperty<Network4>(&Network4::getAuthoritative, authoritative_,
-                                      inheritance, "authoritative"));
+        return (getProperty<Network4>(&Network4::getAuthoritative,
+                                      authoritative_, inheritance,
+                                      CfgGlobals::AUTHORITATIVE));
     }
 
     /// @brief Sets the flag indicating if requests for unknown IP addresses
@@ -1200,7 +1243,7 @@ public:
     util::Optional<asiolink::IOAddress>
     getSiaddr(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network4>(&Network4::getSiaddr, siaddr_,
-                                      inheritance, "next-server"));
+                                      inheritance, CfgGlobals::NEXT_SERVER));
     }
 
     /// @brief Sets server hostname for the network.
@@ -1215,7 +1258,8 @@ public:
     util::Optional<std::string>
     getSname(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network4>(&Network4::getSname, sname_,
-                                      inheritance, "server-hostname"));
+                                      inheritance,
+                                      CfgGlobals::SERVER_HOSTNAME));
     }
 
     /// @brief Sets boot file name for the network.
@@ -1230,7 +1274,8 @@ public:
     util::Optional<std::string>
     getFilename(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network4>(&Network4::getFilename, filename_,
-                                      inheritance, "boot-file-name"));
+                                      inheritance,
+                                      CfgGlobals::BOOT_FILE_NAME));
     }
 
     /// @brief Unparses network object.
@@ -1279,9 +1324,10 @@ public:
     isc::util::Triplet<uint32_t>
     getPreferred(const Inheritance& inheritance = Inheritance::ALL) const {
         return (getProperty<Network6>(&Network6::getPreferred, preferred_,
-                                      inheritance, "preferred-lifetime",
-                                      "min-preferred-lifetime",
-                                      "max-preferred-lifetime"));
+                                      inheritance,
+                                      CfgGlobals::PREFERRED_LIFETIME,
+                                      CfgGlobals::MIN_PREFERRED_LIFETIME,
+                                      CfgGlobals::MAX_PREFERRED_LIFETIME));
     }
 
     /// @brief Sets new preferred lifetime for a network.
@@ -1310,13 +1356,15 @@ public:
     /// @brief Returns boolean value indicating that the Rapid Commit option
     /// is supported or unsupported for the subnet.
     ///
+    /// @note This parameter does not exist at the global level.
+    ///
     /// @param inheritance inheritance mode to be used.
     /// @return true if the Rapid Commit option is supported, false otherwise.
     util::Optional<bool>
     getRapidCommit(const Inheritance& inheritance = Inheritance::ALL) const {
 
         return (getProperty<Network6>(&Network6::getRapidCommit, rapid_commit_,
-                                      inheritance, "rapid-commit"));
+                                      inheritance));
     }
 
     /// @brief Enables or disables Rapid Commit option support for the subnet.
