@@ -795,7 +795,7 @@ public:
 
     /// @brief Sets DHCP4o6 IPC port
     ///
-    /// DHCPv4-over-DHCPv6 uses a UDP socket for interserver communication,
+    /// DHCPv4-over-DHCPv6 uses a UDP socket for inter-server communication,
     /// this socket is bound and connected to this port and port + 1
     ///
     /// @param port port and port + 1 to use
@@ -946,6 +946,24 @@ public:
     /// or disallowed to specify multiple hosts with the same IP reservation.
     void setIPReservationsUnique(const bool unique);
 
+    /// @brief Sets whether the server does host reservations lookup before lease
+    /// lookup.
+    ///
+    /// @param first Boolean value indicating if host reservations lookup should
+    /// be performed before lease lookup.
+    void setReservationsLookupFirst(const bool first) {
+        reservations_lookup_first_ = first;
+    }
+
+    /// @brief Returns whether the server does host reservations lookup before
+    /// lease lookup.
+    ///
+    /// @return Boolean value indicating if host reservations lookup should be
+    /// performed before lease lookup.
+    bool getReservationsLookupFirst() const {
+        return (reservations_lookup_first_);
+    }
+
     /// @brief Unparse a configuration object
     ///
     /// @return a pointer to unparsed configuration
@@ -966,6 +984,21 @@ public:
         return lenient_option_parsing_;
     }
 
+    /// @brief Set ignore RAI Link Selection compatibility flag.
+    ///
+    /// @param value the boolean value to be set when configuring RAI Link
+    /// Selection usage preferences
+    void setIgnoreRAILinkSelection(bool const value) {
+        ignore_rai_link_selection_ = value;
+    }
+
+    /// @brief Get ignore RAI Link Selection compatibility flag.
+    ///
+    /// @return the configured value for RAI Link Selection usage preferences
+    bool getIgnoreRAILinkSelection() const {
+        return ignore_rai_link_selection_;
+    }
+
     /// @brief Convenience method to propagate configuration parameters through
     /// inversion of control.
     ///
@@ -1127,10 +1160,21 @@ private:
     /// @brief Pointer to the configuration consistency settings
     CfgConsistencyPtr cfg_consist_;
 
-    /// @brief Compatibility flags
-    /// @{
+    /// @name Compatibility flags
+    ///
+    //@{
+    /// @brief Indicates whether lenient option parsing is enabled
     bool lenient_option_parsing_;
-    /// @}
+    /// @brief Indicates whether RAI link-selection suboptions will be ignored
+    bool ignore_rai_link_selection_;
+    //@}
+
+    /// @brief Flag which indicates if the server should do host reservations
+    /// lookup before lease lookup. This parameter has effect only when
+    /// multi-threading is disabled. If multi-threading is enabled, host
+    /// reservations lookup is always performed first.
+    /// It default to false when multi-threading is disabled.
+    bool reservations_lookup_first_;
 };
 
 /// @name Pointers to the @c SrvConfig object.
