@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -49,13 +49,14 @@ namespace yang {
 /// YANG syntax for kea-dhcp[46] is with start-address and end-address
 /// as the keys:
 /// @code
-/// +--rw prefix?                  inet:ipv[46]-prefix
-/// +--rw start-address            inet:ipv[46]-address
-/// +--rw end-address              inet:ipv[46]-address
-/// +--rw option-data*
-/// +--rw client-class?            string
-/// +--rw require-client-classes*  string
-/// +--rw user-context?            string
+/// +--rw pool* [start-address end-address]
+///    +--rw prefix?                   inet:ipv[46]-prefix
+///    +--rw start-address             inet:ipv[46]-address
+///    +--rw end-address               inet:ipv[46]-address
+///    +--rw option-data* [code space]
+///    +--rw client-class?             string
+///    +--rw require-client-classes*   string
+///    +--rw user-context?             user-context
 /// @endcode
 ///
 /// An example in JSON and YANG formats for the IETF model:
@@ -128,17 +129,28 @@ public:
     ///
     /// @param session Sysrepo session.
     /// @param model Model name.
-    TranslatorPool(sysrepo::S_Session session, const std::string& model);
+    TranslatorPool(sysrepo::Session session, const std::string& model);
 
     /// @brief Destructor.
     virtual ~TranslatorPool();
 
-    /// @brief Get and translate a pool from YANG to JSON.
+    /// @brief Translate a pool from YANG to JSON.
+    ///
+    /// @param data_node the YANG node representing the pool
+    ///
+    /// @return the JSON representation of the pool
+    ///
+    /// @throw SysrepoError when sysrepo raises an error.
+    isc::data::ElementPtr getPool(libyang::DataNode const& data_node);
+
+    /// @brief Translate a pool from YANG to JSON.
     ///
     /// @param xpath The xpath of the pool.
+    ///
     /// @return JSON representation of the pool.
+    ///
     /// @throw SysrepoError when sysrepo raises an error.
-    isc::data::ElementPtr getPool(const std::string& xpath);
+    isc::data::ElementPtr getPool(std::string const& xpath);
 
     /// @brief Translate and set (address) pool from JSON to YANG.
     ///
@@ -159,10 +171,12 @@ public:
 protected:
     /// @brief getPool for ietf-dhcpv6-server.
     ///
-    /// @param xpath The xpath of the pool.
+    /// @param data_node the YANG node representing the pool configuration
+    ///
     /// @return JSON representation of the pool.
+    ///
     /// @throw BadValue on pool without prefix.
-    isc::data::ElementPtr getPoolIetf6(const std::string& xpath);
+    isc::data::ElementPtr getPoolIetf6(libyang::DataNode const& data_node);
 
     /// @brief setPool for ietf-dhcpv6-server.
     ///
@@ -175,10 +189,12 @@ protected:
 
     /// @brief getPool for kea-dhcp[46]-server.
     ///
-    /// @param xpath The xpath of the pool.
+    /// @param data_node the YANG node representing the pool configuration
+    ///
     /// @return JSON representation of the pool.
+    ///
     /// @throw BadValue on a pool without prefix and start or end address.
-    isc::data::ElementPtr getPoolKea(const std::string& xpath);
+    isc::data::ElementPtr getPoolKea(libyang::DataNode const& data_node);
 
     /// @brief setPool for kea-dhcp[46]-server.
     ///
@@ -198,16 +214,28 @@ public:
     ///
     /// @param session Sysrepo session.
     /// @param model Model name.
-    TranslatorPools(sysrepo::S_Session session, const std::string& model);
+    TranslatorPools(sysrepo::Session session, const std::string& model);
 
     /// @brief Destructor.
     virtual ~TranslatorPools();
 
-    /// @brief Get and translate pools from YANG to JSON.
+    /// @brief Translate pools from YANG to JSON.
+    ///
+    /// @param data_node the YANG node representing the list of pools
+    ///
+    /// @return the JSON representation of the list of pools
+    ///
+    /// @throw SysrepoError when sysrepo raises an error.
+    isc::data::ElementPtr getPools(libyang::DataNode const& data_node);
+
+    /// @brief Translate pools from YANG to JSON.
     ///
     /// @param xpath The xpath of the pool list.
+    ///
+    /// @return the JSON representation of the list of pools
+    ///
     /// @throw SysrepoError when sysrepo raises an error.
-    isc::data::ElementPtr getPools(const std::string& xpath);
+    isc::data::ElementPtr getPools(std::string const& xpath);
 
     /// @brief Translate and set (address) pools from JSON to YANG.
     ///
@@ -218,13 +246,17 @@ public:
 protected:
     /// @brief getPools for ietf-dhcpv6-server.
     ///
-    /// @param xpath The xpath of the pool list.
-    isc::data::ElementPtr getPoolsIetf(const std::string& xpath);
+    /// @param data_node the YANG node representing the list of pools
+    ///
+    /// @return the JSON representation of the list of pools
+    isc::data::ElementPtr getPoolsIetf(libyang::DataNode const& data_node);
 
     /// @brief getPools for kea-dhcp[46]-server.
     ///
-    /// @param xpath The xpath of the pool list.
-    isc::data::ElementPtr getPoolsKea(const std::string& xpath);
+    /// @param data_node the YANG node representing the list of pools
+    ///
+    /// @return the JSON representation of the list of pools
+    isc::data::ElementPtr getPoolsKea(libyang::DataNode const& data_node);
 
     /// @brief setPools using pool-id.
     ///
