@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -14,12 +14,13 @@
 
 using namespace std;
 using namespace isc::data;
+using namespace libyang;
 using namespace sysrepo;
 
 namespace isc {
 namespace yang {
 
-TranslatorSubnet::TranslatorSubnet(S_Session session, const string& model)
+TranslatorSubnet::TranslatorSubnet(Session session, const string& model)
     : TranslatorBasic(session, model),
       TranslatorOptionData(session, model),
       TranslatorOptionDataList(session, model),
@@ -35,50 +36,64 @@ TranslatorSubnet::~TranslatorSubnet() {
 }
 
 ElementPtr
-TranslatorSubnet::getSubnet(const string& xpath) {
+TranslatorSubnet::getSubnet(DataNode const& data_node) {
     try {
         if (model_ == IETF_DHCPV6_SERVER) {
-            return (getSubnetIetf6(xpath));
+            return (getSubnetIetf6(data_node));
         } else if ((model_ == KEA_DHCP4_SERVER) ||
                    (model_ == KEA_DHCP6_SERVER)) {
-            return (getSubnetKea(xpath));
+            return (getSubnetKea(data_node));
         }
-    } catch (const sysrepo_exception& ex) {
+    } catch (Error const& ex) {
         isc_throw(SysrepoError,
-                  "sysrepo error getting subnet at '" << xpath
-                  << "': " << ex.what());
+                  "sysrepo error getting subnet:"
+                  << ex.what());
     }
     isc_throw(NotImplemented,
               "getSubnet not implemented for the model: " << model_);
 }
 
 ElementPtr
-TranslatorSubnet::getSubnetIetf6(const string& xpath) {
+TranslatorSubnet::getSubnet(string const& xpath) {
+    try {
+        return getSubnet(findXPath(xpath));
+    } catch(SysrepoError const&) {
+        return ElementPtr();
+    }
+}
+
+ElementPtr
+TranslatorSubnet::getSubnetIetf6(DataNode const& data_node) {
     ElementPtr result = Element::createMap();
     /// @todo timers
     /// @todo: option-data
-    ConstElementPtr pools = getPools(xpath + "/address-pools");
-    if (pools) {
-        /// Set empty list too.
-        result->set("pools", pools);
+    Set<DataNode> const& address_pools(data_node.findXPath("address-pools"));
+    if (!address_pools.empty()) {
+        ConstElementPtr const& pools(getPools(address_pools.front()));
+        if (pools && !pools->empty()) {
+            result->set("pools", pools);
+        }
     }
-    pools = getPdPools(xpath + "/pd-pools");
-    if (pools && (pools->size() > 0)) {
-        result->set("pd-pools", pools);
+    Set<DataNode> const& yang_pd_pools(data_node.findXPath("pd-pools"));
+    if (!yang_pd_pools.empty()) {
+        ConstElementPtr const& pd_pools(getPdPools(yang_pd_pools.front()));
+        if (pd_pools && !pd_pools->empty()) {
+            result->set("pd-pools", pd_pools);
+        }
     }
-    ConstElementPtr subnet = getItem(xpath + "/network-prefix");
+    ConstElementPtr subnet = getItem(data_node, "network-prefix");
     if (!subnet) {
         isc_throw(BadValue, "getSubnetIetf6 requires network prefix");
     }
     result->set("subnet", subnet);
-    ConstElementPtr id = getItem(xpath + "/network-range-id");
+    ConstElementPtr id = getItem(data_node, "network-range-id");
     if (!id) {
         isc_throw(BadValue, "getSubnetIetf6 requires network range id");
     }
     result->set("id", id);
     /// @todo: reservations
     /// missing a lot of things
-    ConstElementPtr description = getItem(xpath + "/network-description");
+    ConstElementPtr description = getItem(data_node, "network-description");
     /// Adding description if exists.
     if (description) {
         ElementPtr context = Element::createMap();
@@ -93,160 +108,176 @@ TranslatorSubnet::getSubnetIetf6(const string& xpath) {
 }
 
 ElementPtr
-TranslatorSubnet::getSubnetKea(const string& xpath) {
+TranslatorSubnet::getSubnetKea(DataNode const& data_node) {
     ElementPtr result = Element::createMap();
     if (model_ == KEA_DHCP6_SERVER) {
-        ConstElementPtr preferred = getItem(xpath + "/preferred-lifetime");
+        ConstElementPtr preferred = getItem(data_node, "preferred-lifetime");
         if (preferred) {
             result->set("preferred-lifetime", preferred);
         }
-        ConstElementPtr min_pref = getItem(xpath + "/min-preferred-lifetime");
+        ConstElementPtr min_pref = getItem(data_node, "min-preferred-lifetime");
         if (min_pref) {
             result->set("min-preferred-lifetime", min_pref);
         }
-        ConstElementPtr max_pref = getItem(xpath + "/max-preferred-lifetime");
+        ConstElementPtr max_pref = getItem(data_node, "max-preferred-lifetime");
         if (max_pref) {
             result->set("max-preferred-lifetime", max_pref);
         }
     }
-    ConstElementPtr valid = getItem(xpath + "/valid-lifetime");
+    ConstElementPtr valid = getItem(data_node, "valid-lifetime");
     if (valid) {
         result->set("valid-lifetime", valid);
     }
-    ConstElementPtr min_valid = getItem(xpath + "/min-valid-lifetime");
+    ConstElementPtr min_valid = getItem(data_node, "min-valid-lifetime");
     if (min_valid) {
         result->set("min-valid-lifetime", min_valid);
     }
-    ConstElementPtr max_valid = getItem(xpath + "/max-valid-lifetime");
+    ConstElementPtr max_valid = getItem(data_node, "max-valid-lifetime");
     if (max_valid) {
         result->set("max-valid-lifetime", max_valid);
     }
-    ConstElementPtr renew = getItem(xpath + "/renew-timer");
+    ConstElementPtr renew = getItem(data_node, "renew-timer");
 
     if (renew) {
         result->set("renew-timer", renew);
     }
-    ConstElementPtr rebind = getItem(xpath + "/rebind-timer");
+    ConstElementPtr rebind = getItem(data_node, "rebind-timer");
     if (rebind) {
         result->set("rebind-timer", rebind);
     }
-    ConstElementPtr calculate = getItem(xpath + "/calculate-tee-times");
+    ConstElementPtr calculate = getItem(data_node, "calculate-tee-times");
     if (calculate) {
         result->set("calculate-tee-times", calculate);
     }
-    ConstElementPtr t1_percent = getItem(xpath + "/t1-percent");
+    ConstElementPtr t1_percent = getItem(data_node, "t1-percent");
     if (t1_percent) {
         result->set("t1-percent", t1_percent);
     }
-    ConstElementPtr t2_percent = getItem(xpath + "/t2-percent");
+    ConstElementPtr t2_percent = getItem(data_node, "t2-percent");
     if (t2_percent) {
         result->set("t2-percent", t2_percent);
     }
-    ConstElementPtr options = getOptionDataList(xpath);
+    ConstElementPtr options = getOptionDataList(data_node);
     if (options && (options->size() > 0)) {
         result->set("option-data", options);
     }
-    ConstElementPtr pools = getPools(xpath);
+    ConstElementPtr pools = getPools(data_node);
     if (pools && (pools->size() > 0)) {
         result->set("pools", pools);
     }
     if (model_ == KEA_DHCP6_SERVER) {
-        pools = getPdPools(xpath);
-        if (pools && (pools->size() > 0)) {
-            result->set("pd-pools", pools);
+        ElementPtr pd_pools = getPdPools(data_node);
+        if (pd_pools && (pd_pools->size() > 0)) {
+            result->set("pd-pools", pd_pools);
         }
     }
-    ConstElementPtr subnet = getItem(xpath + "/subnet");
+    ConstElementPtr subnet = getItem(data_node, "subnet");
     if (!subnet) {
         isc_throw(BadValue, "getSubnetKea requires subnet");
     }
     result->set("subnet", subnet);
-    ConstElementPtr interface = getItem(xpath + "/interface");
+    ConstElementPtr interface = getItem(data_node, "interface");
     if (interface) {
         result->set("interface", interface);
     }
     if (model_ == KEA_DHCP6_SERVER) {
-        ConstElementPtr interface_id = getItem(xpath + "/interface-id");
+        ConstElementPtr interface_id = getItem(data_node, "interface-id");
         if (interface_id) {
             result->set("interface-id", interface_id);
         }
     }
-    ConstElementPtr id = getItem(xpath + "/id");
+    ConstElementPtr id = getItem(data_node, "id");
     if (!id) {
         isc_throw(BadValue, "getSubnetKea requires id");
     }
     result->set("id", id);
     if (model_ == KEA_DHCP6_SERVER) {
-        ConstElementPtr rapid_commit = getItem(xpath + "/rapid-commit");
+        ConstElementPtr rapid_commit = getItem(data_node, "rapid-commit");
         if (rapid_commit) {
             result->set("rapid-commit", rapid_commit);
         }
     }
-    ConstElementPtr guard =  getItem(xpath + "/client-class");
+    ConstElementPtr guard =  getItem(data_node, "client-class");
     if (guard) {
         result->set("client-class", guard);
     }
-    ConstElementPtr required = getItems(xpath + "/require-client-classes");
+    ConstElementPtr required = getItem(data_node, "require-client-classes");
     if (required && (required->size() > 0)) {
         result->set("require-client-classes", required);
     }
-    ConstElementPtr hosts = getHosts(xpath);
+    ConstElementPtr hosts = getHosts(data_node);
     if (hosts && (hosts->size() > 0)) {
         result->set("reservations", hosts);
     }
-    ConstElementPtr mode = getItem(xpath + "/reservation-mode");
+    ConstElementPtr mode = getItem(data_node, "reservation-mode");
     if (mode) {
         result->set("reservation-mode", mode);
     }
-    ConstElementPtr relay = getItems(xpath + "/relay/ip-addresses");
+    ConstElementPtr relay = getItem(data_node, "relay/ip-addresses");
     if (relay && (relay->size() > 0)) {
         ElementPtr relay_map = Element::createMap();
         relay_map->set("ip-addresses", relay);
         result->set("relay", relay_map);
     }
     if (model_ == KEA_DHCP4_SERVER) {
-        ConstElementPtr match = getItem(xpath + "/match-client-id");
+        ConstElementPtr match = getItem(data_node, "match-client-id");
         if (match) {
             result->set("match-client-id", match);
         }
-        ConstElementPtr auth = getItem(xpath + "/authoritative");
+        ConstElementPtr auth = getItem(data_node, "authoritative");
         if (auth) {
             result->set("authoritative", auth);
         }
-        ConstElementPtr next = getItem(xpath + "/next-server");
+        ConstElementPtr next = getItem(data_node, "next-server");
         if (next) {
             result->set("next-server", next);
         }
-        ConstElementPtr hostname = getItem(xpath + "/server-hostname");
+        ConstElementPtr hostname = getItem(data_node, "server-hostname");
         if (hostname) {
             result->set("server-hostname", hostname);
         }
-        ConstElementPtr boot = getItem(xpath + "/boot-file-name");
+        ConstElementPtr boot = getItem(data_node, "boot-file-name");
         if (boot) {
             result->set("boot-file-name", boot);
         }
-        ConstElementPtr s4o6_if = getItem(xpath + "/subnet-4o6-interface");
+        ConstElementPtr s4o6_if = getItem(data_node, "subnet-4o6-interface");
         if (s4o6_if) {
             result->set("4o6-interface", s4o6_if);
         }
-        ConstElementPtr s4o6_id = getItem(xpath + "/subnet-4o6-interface-id");
+        ConstElementPtr s4o6_id = getItem(data_node, "subnet-4o6-interface-id");
         if (s4o6_id) {
             result->set("4o6-interface-id", s4o6_id);
         }
-        ConstElementPtr s4o6_sub = getItem(xpath + "/subnet-4o6-subnet");
+        ConstElementPtr s4o6_sub = getItem(data_node, "subnet-4o6-subnet");
         if (s4o6_sub) {
             result->set("4o6-subnet", s4o6_sub);
         }
     }
-    ConstElementPtr context = getItem(xpath + "/user-context");
+    ConstElementPtr context = getItem(data_node, "user-context");
     if (context) {
         result->set("user-context", Element::fromJSON(context->stringValue()));
     }
+    checkAndGetLeaf(result, data_node, "cache-max-age");
+    checkAndGetLeaf(result, data_node, "cache-threshold");
+    checkAndGetLeaf(result, data_node, "ddns-generated-prefix");
+    checkAndGetLeaf(result, data_node, "ddns-override-client-update");
+    checkAndGetLeaf(result, data_node, "ddns-override-no-update");
+    checkAndGetLeaf(result, data_node, "ddns-qualifying-suffix");
+    checkAndGetLeaf(result, data_node, "ddns-replace-client-name");
+    checkAndGetLeaf(result, data_node, "ddns-send-updates");
+    checkAndGetLeaf(result, data_node, "ddns-update-on-renew");
+    checkAndGetLeaf(result, data_node, "ddns-use-conflict-resolution");
+    checkAndGetLeaf(result, data_node, "hostname-char-replacement");
+    checkAndGetLeaf(result, data_node, "hostname-char-set");
+    checkAndGetLeaf(result, data_node, "reservations-global");
+    checkAndGetLeaf(result, data_node, "reservations-in-subnet");
+    checkAndGetLeaf(result, data_node, "reservations-out-of-pool");
+    checkAndGetLeaf(result, data_node, "store-extended-info");
     return (result);
 }
 
 void
-TranslatorSubnet::setSubnet(const string& xpath, ConstElementPtr elem) {
+TranslatorSubnet::setSubnet(string const& xpath, ConstElementPtr elem) {
     try {
         if (model_ == IETF_DHCPV6_SERVER) {
             setSubnetIetf6(xpath, elem);
@@ -257,29 +288,29 @@ TranslatorSubnet::setSubnet(const string& xpath, ConstElementPtr elem) {
             isc_throw(NotImplemented,
                       "setSubnet not implemented for the model: " << model_);
         }
-    } catch (const sysrepo_exception& ex) {
+    } catch (Error const& ex) {
         isc_throw(SysrepoError,
                   "sysrepo error setting subnet '" << elem->str()
-                  << "' at '" << xpath << "': " << ex.what());
+                  << "' : " << ex.what());
     }
 }
 
 void
-TranslatorSubnet::setSubnetIetf6(const string& xpath, ConstElementPtr elem) {
+TranslatorSubnet::setSubnetIetf6(string const& xpath, ConstElementPtr elem) {
     /// Skip id as it is the key.
     AdaptorPool::fromSubnet(model_, elem, elem->get("pools"));
     ConstElementPtr context = elem->get("user-context");
     if (context && context->contains("description")) {
         ConstElementPtr description = context->get("description");
         if (description->getType() == Element::string) {
-            setItem(xpath + "/network-description", description, SR_STRING_T);
+            setItem(xpath + "/network-description", description, LeafBaseType::String);
         }
     }
     ConstElementPtr subnet = elem->get("subnet");
     if (!subnet) {
         isc_throw(BadValue, "setSubnetIetf6 requires subnet: " << elem->str());
     }
-    setItem(xpath + "/network-prefix", subnet, SR_STRING_T);
+    setItem(xpath + "/network-prefix", subnet, LeafBaseType::String);
     /// @todo option-data
     ConstElementPtr pools = elem->get("pools");
     if (pools && (pools->size() > 0)) {
@@ -293,53 +324,53 @@ TranslatorSubnet::setSubnetIetf6(const string& xpath, ConstElementPtr elem) {
 }
 
 void
-TranslatorSubnet::setSubnetKea(const string& xpath, ConstElementPtr elem) {
+TranslatorSubnet::setSubnetKea(string const& xpath, ConstElementPtr elem) {
     /// Skip id as it is the key.
     if (model_ == KEA_DHCP6_SERVER) {
         ConstElementPtr preferred = elem->get("preferred-lifetime");
         if (preferred) {
-            setItem(xpath + "/preferred-lifetime", preferred, SR_UINT32_T);
+            setItem(xpath + "/preferred-lifetime", preferred, LeafBaseType::Uint32);
         }
         ConstElementPtr min_pref = elem->get("min-preferred-lifetime");
         if (min_pref) {
-            setItem(xpath + "/min-preferred-lifetime", min_pref, SR_UINT32_T);
+            setItem(xpath + "/min-preferred-lifetime", min_pref, LeafBaseType::Uint32);
         }
         ConstElementPtr max_pref = elem->get("max-preferred-lifetime");
         if (max_pref) {
-            setItem(xpath + "/max-preferred-lifetime", max_pref, SR_UINT32_T);
+            setItem(xpath + "/max-preferred-lifetime", max_pref, LeafBaseType::Uint32);
         }
     }
     ConstElementPtr valid = elem->get("valid-lifetime");
     if (valid) {
-        setItem(xpath + "/valid-lifetime", valid, SR_UINT32_T);
+        setItem(xpath + "/valid-lifetime", valid, LeafBaseType::Uint32);
     }
     ConstElementPtr min_valid = elem->get("min-valid-lifetime");
     if (min_valid) {
-        setItem(xpath + "/min-valid-lifetime", min_valid, SR_UINT32_T);
+        setItem(xpath + "/min-valid-lifetime", min_valid, LeafBaseType::Uint32);
     }
     ConstElementPtr max_valid = elem->get("max-valid-lifetime");
     if (max_valid) {
-        setItem(xpath + "/max-valid-lifetime", max_valid, SR_UINT32_T);
+        setItem(xpath + "/max-valid-lifetime", max_valid, LeafBaseType::Uint32);
     }
     ConstElementPtr renew = elem->get("renew-timer");
     if (renew) {
-        setItem(xpath + "/renew-timer", renew, SR_UINT32_T);
+        setItem(xpath + "/renew-timer", renew, LeafBaseType::Uint32);
     }
     ConstElementPtr rebind = elem->get("rebind-timer");
     if (rebind) {
-        setItem(xpath + "/rebind-timer", rebind, SR_UINT32_T);
+        setItem(xpath + "/rebind-timer", rebind, LeafBaseType::Uint32);
     }
     ConstElementPtr calculate = elem->get("calculate-tee-times");
     if (calculate) {
-        setItem(xpath + "/calculate-tee-times", calculate, SR_BOOL_T);
+        setItem(xpath + "/calculate-tee-times", calculate, LeafBaseType::Bool);
     }
     ConstElementPtr t1_percent =  elem->get("t1-percent");
     if (t1_percent) {
-        setItem(xpath + "/t1-percent", t1_percent, SR_DECIMAL64_T);
+        setItem(xpath + "/t1-percent", t1_percent, LeafBaseType::Dec64);
     }
     ConstElementPtr t2_percent =  elem->get("t2-percent");
     if (t2_percent) {
-        setItem(xpath + "/t2-percent", t2_percent, SR_DECIMAL64_T);
+        setItem(xpath + "/t2-percent", t2_percent, LeafBaseType::Dec64);
     }
     ConstElementPtr options = elem->get("option-data");
     if (options && (options->size() > 0)) {
@@ -359,31 +390,31 @@ TranslatorSubnet::setSubnetKea(const string& xpath, ConstElementPtr elem) {
     if (!subnet) {
         isc_throw(BadValue, "setSubnetKea requires subnet: " << elem->str());
     }
-    setItem(xpath + "/subnet", subnet, SR_STRING_T);
+    setItem(xpath + "/subnet", subnet, LeafBaseType::String);
     ConstElementPtr interface = elem->get("interface");
     if (interface) {
-        setItem(xpath + "/interface", interface, SR_STRING_T);
+        setItem(xpath + "/interface", interface, LeafBaseType::String);
     }
     if (model_ == KEA_DHCP6_SERVER) {
         ConstElementPtr interface_id = elem->get("interface-id");
         if (interface_id) {
-            setItem(xpath + "/interface-id", interface_id, SR_STRING_T);
+            setItem(xpath + "/interface-id", interface_id, LeafBaseType::String);
         }
     }
     if (model_ == KEA_DHCP6_SERVER) {
         ConstElementPtr rapid_commit = elem->get("rapid-commit");
         if (rapid_commit) {
-            setItem(xpath + "/rapid-commit", rapid_commit, SR_BOOL_T);
+            setItem(xpath + "/rapid-commit", rapid_commit, LeafBaseType::Bool);
         }
     }
     ConstElementPtr guard = elem->get("client-class");
     if (guard) {
-        setItem(xpath + "/client-class", guard, SR_STRING_T);
+        setItem(xpath + "/client-class", guard, LeafBaseType::String);
     }
     ConstElementPtr required = elem->get("require-client-classes");
     if (required && (required->size() > 0)) {
         for (ConstElementPtr rclass : required->listValue()) {
-            setItem(xpath + "/require-client-classes", rclass, SR_STRING_T);
+            setItem(xpath + "/require-client-classes", rclass, LeafBaseType::String);
         }
     }
     ConstElementPtr hosts = elem->get("reservations");
@@ -392,63 +423,78 @@ TranslatorSubnet::setSubnetKea(const string& xpath, ConstElementPtr elem) {
     }
     ConstElementPtr mode = elem->get("reservation-mode");
     if (mode) {
-        setItem(xpath + "/reservation-mode", mode, SR_ENUM_T);
+        setItem(xpath + "/reservation-mode", mode, LeafBaseType::Enum);
     }
     ConstElementPtr relay = elem->get("relay");
     if (relay) {
         ConstElementPtr address = relay->get("ip-address");
         ConstElementPtr addresses = relay->get("ip-addresses");
         if (address) {
-            setItem(xpath + "/relay/ip-addresses", address, SR_STRING_T);
+            setItem(xpath + "/relay/ip-addresses", address, LeafBaseType::String);
         } else if (addresses && (addresses->size() > 0)) {
             for (ConstElementPtr addr : addresses->listValue()) {
-                setItem(xpath + "/relay/ip-addresses", addr, SR_STRING_T);
+                setItem(xpath + "/relay/ip-addresses", addr, LeafBaseType::String);
             }
         }
     }
-    checkAndSetLeaf(elem, xpath, "store-extended-info", SR_BOOL_T);
+    checkAndSetLeaf(elem, xpath, "cache-max-age", LeafBaseType::Uint32);
+    checkAndSetLeaf(elem, xpath, "cache-threshold", LeafBaseType::Dec64);
+    checkAndSetLeaf(elem, xpath, "ddns-generated-prefix", LeafBaseType::String);
+    checkAndSetLeaf(elem, xpath, "ddns-override-client-update", LeafBaseType::Bool);
+    checkAndSetLeaf(elem, xpath, "ddns-override-no-update", LeafBaseType::Bool);
+    checkAndSetLeaf(elem, xpath, "ddns-qualifying-suffix", LeafBaseType::String);
+    checkAndSetLeaf(elem, xpath, "ddns-replace-client-name", LeafBaseType::String);
+    checkAndSetLeaf(elem, xpath, "ddns-send-updates", LeafBaseType::Bool);
+    checkAndSetLeaf(elem, xpath, "ddns-update-on-renew", LeafBaseType::Bool);
+    checkAndSetLeaf(elem, xpath, "ddns-use-conflict-resolution", LeafBaseType::Bool);
+    checkAndSetLeaf(elem, xpath, "hostname-char-replacement", LeafBaseType::String);
+    checkAndSetLeaf(elem, xpath, "hostname-char-set", LeafBaseType::String);
+    checkAndSetLeaf(elem, xpath, "reservations-global", LeafBaseType::Bool);
+    checkAndSetLeaf(elem, xpath, "reservations-in-subnet", LeafBaseType::Bool);
+    checkAndSetLeaf(elem, xpath, "reservations-out-of-pool", LeafBaseType::Bool);
+    checkAndSetLeaf(elem, xpath, "store-extended-info", LeafBaseType::Bool);
     if (model_ == KEA_DHCP4_SERVER) {
         ConstElementPtr match = elem->get("match-client-id");
         if (match) {
-            setItem(xpath + "/match-client-id", match, SR_BOOL_T);
+            setItem(xpath + "/match-client-id", match, LeafBaseType::Bool);
         }
         ConstElementPtr auth = elem->get("authoritative");
         if (auth) {
-            setItem(xpath + "/authoritative", auth, SR_BOOL_T);
+            setItem(xpath + "/authoritative", auth, LeafBaseType::Bool);
         }
         ConstElementPtr next = elem->get("next-server");
         if (next) {
-            setItem(xpath + "/next-server", next, SR_STRING_T);
+            setItem(xpath + "/next-server", next, LeafBaseType::String);
         }
         ConstElementPtr hostname = elem->get("server-hostname");
         if (hostname) {
-            setItem(xpath + "/server-hostname", hostname, SR_STRING_T);
+            setItem(xpath + "/server-hostname", hostname, LeafBaseType::String);
         }
         ConstElementPtr boot = elem->get("boot-file-name");
         if (boot) {
-            setItem(xpath + "/boot-file-name", boot, SR_STRING_T);
+            setItem(xpath + "/boot-file-name", boot, LeafBaseType::String);
         }
         ConstElementPtr s4o6_if = elem->get("4o6-interface");
         if (s4o6_if) {
-            setItem(xpath + "/subnet-4o6-interface", s4o6_if, SR_STRING_T);
+            setItem(xpath + "/subnet-4o6-interface", s4o6_if, LeafBaseType::String);
         }
         ConstElementPtr s4o6_id = elem->get("4o6-interface-id");
         if (s4o6_id) {
-            setItem(xpath + "/subnet-4o6-interface-id", s4o6_id, SR_STRING_T);
+            setItem(xpath + "/subnet-4o6-interface-id", s4o6_id, LeafBaseType::String);
         }
         ConstElementPtr s4o6_subnet = elem->get("4o6-subnet");
         if (s4o6_subnet) {
-            setItem(xpath + "/subnet-4o6-subnet", s4o6_subnet, SR_STRING_T);
+            setItem(xpath + "/subnet-4o6-subnet", s4o6_subnet, LeafBaseType::String);
         }
     }
     ConstElementPtr context = Adaptor::getContext(elem);
     if (context) {
         ConstElementPtr repr = Element::create(context->str());
-        setItem(xpath + "/user-context", repr, SR_STRING_T);
+        setItem(xpath + "/user-context", repr, LeafBaseType::String);
     }
 }
 
-TranslatorSubnets::TranslatorSubnets(S_Session session, const string& model)
+TranslatorSubnets::TranslatorSubnets(Session session, const string& model)
     : TranslatorBasic(session, model),
       TranslatorOptionData(session, model),
       TranslatorOptionDataList(session, model),
@@ -465,33 +511,42 @@ TranslatorSubnets::~TranslatorSubnets() {
 }
 
 ElementPtr
-TranslatorSubnets::getSubnets(const string& xpath) {
+TranslatorSubnets::getSubnets(DataNode const& data_node) {
     try {
         if (model_ == IETF_DHCPV6_SERVER) {
-            return (getSubnetsCommon(xpath, "network-range"));
+            return (getSubnetsCommon(data_node, "network-range"));
         } else if (model_ == KEA_DHCP4_SERVER) {
-            return (getSubnetsCommon(xpath, "subnet4"));
+            return (getSubnetsCommon(data_node, "subnet4"));
         } else if (model_ == KEA_DHCP6_SERVER) {
-            return (getSubnetsCommon(xpath, "subnet6"));
+            return (getSubnetsCommon(data_node, "subnet6"));
         }
-    } catch (const sysrepo_exception& ex) {
+    } catch (Error const& ex) {
         isc_throw(SysrepoError,
-                  "sysrepo error getting subnets at '" << xpath
-                  << "': " << ex.what());
+                  "sysrepo error getting subnets:"
+                  << ex.what());
     }
     isc_throw(NotImplemented,
               "getSubnets not implemented for the model: " << model_);
 }
 
 ElementPtr
-TranslatorSubnets::getSubnetsCommon(const string& xpath,
+TranslatorSubnets::getSubnets(string const& xpath) {
+    try {
+        return getSubnets(findXPath(xpath));
+    } catch(SysrepoError const&) {
+        return ElementPtr();
+    }
+}
+
+ElementPtr
+TranslatorSubnets::getSubnetsCommon(DataNode const& data_node,
                                     const std::string& subsel) {
-    return getList<TranslatorSubnet>(xpath + "/" + subsel, *this,
+    return getList<TranslatorSubnet>(data_node, subsel, *this,
                                      &TranslatorSubnet::getSubnet);
 }
 
 void
-TranslatorSubnets::setSubnets(const string& xpath, ConstElementPtr elem) {
+TranslatorSubnets::setSubnets(string const& xpath, ConstElementPtr elem) {
     try {
         if (model_ == IETF_DHCPV6_SERVER) {
             setSubnetsIetf6(xpath, elem);
@@ -503,15 +558,15 @@ TranslatorSubnets::setSubnets(const string& xpath, ConstElementPtr elem) {
             isc_throw(NotImplemented,
                       "setSubnets not implemented for the model: " << model_);
         }
-    } catch (const sysrepo_exception& ex) {
+    } catch (Error const& ex) {
         isc_throw(SysrepoError,
                   "sysrepo error setting subnets '" << elem->str()
-                  << "' at '" << xpath << "': " << ex.what());
+                  << "' : " << ex.what());
     }
 }
 
 void
-TranslatorSubnets::setSubnetsIetf6(const string& xpath, ConstElementPtr elem) {
+TranslatorSubnets::setSubnetsIetf6(string const& xpath, ConstElementPtr elem) {
     for (size_t i = 0; i < elem->size(); ++i) {
         ConstElementPtr subnet = elem->get(i);
         ostringstream range;
@@ -521,12 +576,12 @@ TranslatorSubnets::setSubnetsIetf6(const string& xpath, ConstElementPtr elem) {
             isc_throw(BadValue, "subnet without id: " << elem->str());
         }
         range << id->intValue() << "']";
-        setSubnet(range.str().c_str(), subnet);
+        setSubnet(range.str(), subnet);
     }
 }
 
 void
-TranslatorSubnets::setSubnetsKea(const string& xpath, ConstElementPtr elem,
+TranslatorSubnets::setSubnetsKea(string const& xpath, ConstElementPtr elem,
                                  const std::string& subsel) {
     for (size_t i = 0; i < elem->size(); ++i) {
         ConstElementPtr subnet = elem->get(i);
